<?php

namespace FiloBlu\Esb\Model;

use DateTime;
use FiloBlu\Esb\Api\Data\MessageInterface;
use FiloBlu\Esb\Api\Data\MessageInterfaceFactory;
use FiloBlu\Esb\Api\Data\QueueItemInterface;
use FiloBlu\Esb\Api\Data\QueueItemInterfaceFactory;
use FiloBlu\Esb\Api\Data\StatusInterface;
use FiloBlu\Esb\Api\Data\StatusInterfaceFactory;
use FiloBlu\Esb\Api\QueueItemSqlRepositoryInterface;
use Magento\Framework\Api\Search\SearchCriteriaInterfaceFactory;
use Magento\Framework\Api\SearchResultsInterfaceFactory;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DataObject;

/**
 * Class QueueItemRepository
 * @package FiloBlu\Esb\Model
 */
class QueueItemRepository extends AbstractSqlRepository implements QueueItemSqlRepositoryInterface
{

    /**
     * @var QueueItemInterfaceFactory
     */
    private $queueItemFactory;

    /**
     * @var MessageInterfaceFactory
     */
    private $messageFactory;
    /**
     * @var StatusInterfaceFactory
     */
    private $statusInterfaceFactory;

    /**
     * QueueItemRepository constructor.
     * @param ResourceConnection $resourceConnection
     * @param SearchCriteriaInterfaceFactory $searchCriteriaFactory
     * @param SearchResultsInterfaceFactory $searchResultsFactory
     * @param QueueItemInterfaceFactory $queueItemFactory
     * @param MessageInterfaceFactory $messageFactory
     * @param StatusInterfaceFactory $statusInterfaceFactory
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        SearchCriteriaInterfaceFactory $searchCriteriaFactory,
        SearchResultsInterfaceFactory $searchResultsFactory,
        QueueItemInterfaceFactory $queueItemFactory,
        MessageInterfaceFactory $messageFactory,
        StatusInterfaceFactory $statusInterfaceFactory
    ) {
        parent::__construct($resourceConnection, $searchCriteriaFactory, $searchResultsFactory);
        $this->queueItemFactory = $queueItemFactory;
        $this->messageFactory = $messageFactory;
        $this->statusInterfaceFactory = $statusInterfaceFactory;
    }

    /**
     * @param array $item
     * @return QueueItemInterface
     */
    public function fromRow(array $item)
    {
        $publishedAt = DateTime::createFromFormat(self::DATE_FORMAT, $item[QueueItemInterface::PUBLISHED_AT] ?? '');

        if (!$publishedAt) {
            $publishedAt = null;
        }

        $executableAfter = DateTime::createFromFormat(self::DATE_FORMAT, $item[QueueItemInterface::EXECUTABLE_AFTER] ?? '');

        if (!$executableAfter) {
            $executableAfter = null;
        }

        $executedAt = DateTime::createFromFormat(self::DATE_FORMAT, $item[QueueItemInterface::EXECUTED_AT] ?? '');

        if (!$executedAt) {
            $executedAt = null;
        }

        $finishedAt = DateTime::createFromFormat(self::DATE_FORMAT, $item[QueueItemInterface::FINISHED_AT] ?? '');

        if (!$finishedAt) {
            $finishedAt = null;
        }

        $queueItem = $this->queueItemFactory->create();

        $status = $this->statusInterfaceFactory->create();

        $status
            ->setCode($item[QueueItemInterface::STATUS])
            ->setOutputData($item[StatusInterface::STATUS_OUTPUT])
            ->setInputData($item[StatusInterface::STATUS_INPUT]);

        /** @var MessageInterface $message */
        $message = $this->messageFactory->create();
        $message->setPayload(new DataObject(json_decode($item[QueueItemInterface::MESSAGE], true)));

        /** @var QueueItemInterface $queueItem */
        $queueItem
            ->setId($item[QueueItemInterface::ID])
            ->setRetryCount($item[QueueItemInterface::RETRY_COUNT])
            ->setMessage($message)
            ->setStatus($status)
            ->setRetryItemId($item[QueueItemInterface::RETRY_ITEM_ID])
            ->setEvent($item[QueueItemInterface::EVENT])
            ->setPriority($item[QueueItemInterface::PRIORITY])
            ->setTo($item[QueueItemInterface::TO])
            ->setFrom($item[QueueItemInterface::FROM])
            ->setExecutableAfter($executableAfter)
            ->setExecutedAt($executedAt)
            ->setFinishedAt($finishedAt)
            ->setPublishedAt($publishedAt);

        return $queueItem;
    }

    /**
     * @return string
     */
    public function getIdFieldName(): string
    {
        return QueueItemInterface::ID;
    }

    /**
     * @param QueueItemInterface $item
     * @return QueueItemInterface
     */
    public function save(QueueItemInterface $item)
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $connection->getTableName(QueueItemSqlRepositoryInterface::TABLE);

        $bindings = [
            QueueItemInterface::EVENT => $item->getEvent(),
            QueueItemInterface::PRIORITY => $item->getPriority(),
            QueueItemInterface::MESSAGE => $item->getMessage()->getPayload() ? $item->getMessage()->getPayload()->toJson() : '{}',
            QueueItemInterface::EXECUTABLE_AFTER => $item->getExecutableAfter() ? $item->getExecutableAfter()->format(self::DATE_FORMAT) : null,
            QueueItemInterface::FINISHED_AT => $item->getFinishedAt() ? $item->getFinishedAt()->format(self::DATE_FORMAT) : null,
            QueueItemInterface::EXECUTED_AT => $item->getExecutedAt() ? $item->getExecutedAt()->format(self::DATE_FORMAT) : null,
            QueueItemInterface::PUBLISHED_AT => $item->getPublishedAt() ? $item->getPublishedAt()->format(self::DATE_FORMAT) : null,
            QueueItemInterface::RETRY_COUNT => $item->getRetryCount(),
            QueueItemInterface::RETRY_ITEM_ID => $item->getRetryItemId(),
            QueueItemInterface::FROM => $item->getFrom(),
            QueueItemInterface::TO => $item->getTo(),
            QueueItemInterface::STATUS => $item->getStatus() ? $item->getStatus()->getCode() : 'pending',
            StatusInterface::STATUS_INPUT => $item->getStatus()->getInputData(),
            StatusInterface::STATUS_OUTPUT => $item->getStatus()->getOutputData()

        ];

        if ($item->getId()) {
            $connection->update($table, $bindings, $connection->quoteInto(
                sprintf('%s = ?', QueueItemInterface::ID),
                $item->getId()
            ));
        } else {
            $connection->insert($table, $bindings);
            $item->setId($connection->lastInsertId($table));
        }

        return $item;
    }

    /**
     * @param QueueItemInterface $item
     * @return mixed
     */
    public function delete(QueueItemInterface $item)
    {
        if (!($id = $item->getId())) {
            return;
        }

        $connection = $this->resourceConnection->getConnection();
        $table = $connection->getTableName(QueueItemSqlRepositoryInterface::TABLE);

        $connection->delete($table, [sprintf('%s = ?', QueueItemInterface::ID) => $id]);
    }

    /**
     * @inheritDoc
     */
    public function getTable(): string
    {
        return self::TABLE;
    }

    /**
     * @inheritDoc
     */
    public function isPending(QueueItemInterface $item): bool
    {
        $queue_item_field_id = QueueItemInterface::ID;
        $queue_item_field_status = QueueItemInterface::STATUS;
        $table_name = QueueItemSqlRepositoryInterface::TABLE;
        $status_pending = StatusInterface::PENDING;
        $conn = $this->resourceConnection->getConnection();
        $conn->beginTransaction();
        try {
            $sql = "SELECT {$queue_item_field_id} FROM {$table_name} WHERE {$queue_item_field_id} = {$item->getId()} AND {$queue_item_field_status} = '{$status_pending}' FOR UPDATE";
            $result = $conn->query($sql)->fetch();
            if ($result === false){
                $conn->commit();
                return false;
            }
            $conn->update($table_name,array($queue_item_field_status => StatusInterface::PROCESSING),"{$queue_item_field_id} = {$item->getId()}");
            $conn->commit();
            return true;
        } catch (\Throwable $e) {
            $conn->rollBack();
            return false;
        }
    }
}
